@@ -407,7 +407,8 @@ hidpp10drv_read_profile(struct ratbag_profile *profile)
 	if (rc)
 		return;
 
-	ratbag_profile_set_cap(profile, RATBAG_PROFILE_CAP_WRITABLE_NAME);
+	if (hidpp10->profile_type != HIDPP10_PROFILE_UNKNOWN)
+		ratbag_profile_set_cap(profile, RATBAG_PROFILE_CAP_DISABLE);
 
 	profile->is_enabled = p.enabled;
 
@@ -423,16 +424,12 @@ hidpp10drv_read_profile(struct ratbag_profile *profile)
 	if (rc)
 		xres = 0xffff;
 
-	ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_RESOLUTION);
-	ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_SWITCHABLE_RESOLUTION);
-
 	ratbag_profile_for_each_resolution(profile, res) {
 		unsigned int dpis[hidpp10->dpi_count];
 
 		ratbag_resolution_set_resolution(res,
 						 p.dpi_modes[res->index].xres,
-						 p.dpi_modes[res->index].yres,
-						 p.refresh_rate);
+						 p.dpi_modes[res->index].yres);
 		ratbag_resolution_set_cap(res,
 					  RATBAG_RESOLUTION_CAP_SEPARATE_XY_RESOLUTION);
 		if (profile->is_active &&
@@ -453,17 +450,18 @@ hidpp10drv_read_profile(struct ratbag_profile *profile)
 			/* FIXME: this relies on libratbag using the
 			 * same steps that we support */
 			ratbag_resolution_set_dpi_list_from_range(res, min, max);
-		} else {
-			for (int i = 0; i < hidpp10->dpi_count; i++)
+		} else if (hidpp10->dpi_count > 0) {
+			for (uint8_t i = 0; i < hidpp10->dpi_count; i++)
 				dpis[i] = hidpp10->dpi_table[i].dpi;
 
 			ratbag_resolution_set_dpi_list(res, dpis, hidpp10->dpi_count);
 		}
 
-		ratbag_resolution_set_report_rate_list(res, rates,
-						       ARRAY_LENGTH(rates));
 	}
 
+	ratbag_profile_set_report_rate_list(profile, rates, ARRAY_LENGTH(rates));
+	ratbag_profile_set_report_rate(profile, p.refresh_rate);
+
 	ratbag_profile_for_each_button(profile, button)
 		hidpp10drv_read_button(button);
 
@@ -501,15 +499,6 @@ hidpp10drv_fill_from_profile(struct ratbag_device *device, struct hidpp10_device
 				    profile.num_buttons,
 				    profile.num_leds);
 
-	if (dev->profile_type != HIDPP10_PROFILE_UNKNOWN) {
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_PROFILE);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_SWITCHABLE_PROFILE);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_DISABLE_PROFILE);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON_KEY);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON_MACROS);
-	}
-
 	return 0;
 }
 
