@@ -805,7 +805,7 @@ hidpp20drv_read_resolution_dpi(struct ratbag_profile *profile)
 			sensor = &drv_data->sensors[0];
 
 			/* FIXME: retrieve the refresh rate */
-			ratbag_resolution_set_resolution(res, sensor->dpi, sensor->dpi, 0);
+			ratbag_resolution_set_resolution(res, sensor->dpi, sensor->dpi);
 			ratbag_resolution_set_dpi_list_from_range(res,
 								  sensor->dpi_min,
 								  sensor->dpi_max);
@@ -823,13 +823,11 @@ hidpp20drv_read_resolution_dpi(struct ratbag_profile *profile)
 		if (rc < 0)
 			return rc;
 
-		ratbag_profile_for_each_resolution(profile, res)
-			ratbag_resolution_set_report_rate_list(res,
-							       drv_data->report_rates,
-							       drv_data->num_report_rates);
+		ratbag_profile_set_report_rate_list(profile,
+						    drv_data->report_rates,
+						    drv_data->num_report_rates);
 	} else {
-		ratbag_profile_for_each_resolution(profile, res)
-			ratbag_resolution_set_report_rate_list(res, &default_rate, 1);
+		ratbag_profile_set_report_rate_list(profile, &default_rate, 1);
 	}
 
 	return 0;
@@ -1017,8 +1015,7 @@ hidpp20drv_read_profile_8100(struct ratbag_profile *profile)
 
 		ratbag_resolution_set_resolution(res,
 						 p->dpi[res->index],
-						 p->dpi[res->index],
-						 p->report_rate);
+						 p->dpi[res->index]);
 
 		if (profile->is_active &&
 		    res->index == (unsigned int)dpi_index)
@@ -1032,10 +1029,12 @@ hidpp20drv_read_profile_8100(struct ratbag_profile *profile)
 		ratbag_resolution_set_dpi_list_from_range(res,
 							  sensor->dpi_min,
 							  sensor->dpi_max);
-		ratbag_resolution_set_report_rate_list(res,
-						       drv_data->report_rates,
-						       drv_data->num_report_rates);
 	}
+
+	ratbag_profile_set_report_rate_list(profile,
+					    drv_data->report_rates,
+					    drv_data->num_report_rates);
+	ratbag_profile_set_report_rate(profile, p->report_rate);
 }
 
 static void
@@ -1048,6 +1047,7 @@ hidpp20drv_read_profile(struct ratbag_profile *profile)
 
 	if (drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100) {
 		hidpp20drv_read_profile_8100(profile);
+		ratbag_profile_set_cap(profile, RATBAG_PROFILE_CAP_DISABLE);
 	} else {
 		hidpp20drv_read_resolution_dpi(profile);
 		hidpp20drv_read_special_key_mouse(device);
@@ -1085,7 +1085,6 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 		break;
 	case HIDPP_PAGE_MOUSE_POINTER_BASIC: {
 		drv_data->capabilities |= HIDPP_CAP_RESOLUTION_2200;
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_RESOLUTION);
 		break;
 	}
 	case HIDPP_PAGE_ADJUSTABLE_DPI: {
@@ -1095,16 +1094,12 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 		rc = hidpp20drv_read_resolution_dpi_2201(device);
 		if (rc < 0)
 			return 0; /* this is not a hard failure */
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_RESOLUTION);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_SWITCHABLE_RESOLUTION);
 		drv_data->capabilities |= HIDPP_CAP_SWITCHABLE_RESOLUTION_2201;
 		break;
 	}
 	case HIDPP_PAGE_SPECIAL_KEYS_BUTTONS: {
 		log_debug(ratbag, "device has programmable keys/buttons\n");
 		drv_data->capabilities |= HIDPP_CAP_BUTTON_KEY_1b04;
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON_KEY);
 		/* we read the profile once to get the correct number of
 		 * supported buttons. */
 		if (!hidpp20drv_read_special_key_mouse(device))
@@ -1157,7 +1152,6 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 		if (hidpp20drv_read_color_leds(device))
 			return 0;
 
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_LED);
 		device->num_leds = drv_data->num_leds;
 		break;
 	}
@@ -1171,17 +1165,11 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 		if (hidpp20drv_read_leds(device))
 			return 0;
 
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_LED);
 		break;
 	}
 	case HIDPP_PAGE_ONBOARD_PROFILES: {
 		log_debug(ratbag, "device has onboard profiles\n");
 		drv_data->capabilities |= HIDPP_CAP_ONBOARD_PROFILES_8100;
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_PROFILE);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_DISABLE_PROFILE);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON_KEY);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON_MACROS);
 
 		rc = hidpp20_onboard_profiles_allocate(drv_data->dev, &drv_data->profiles);
 		if (rc < 0)
