@@ -188,13 +188,6 @@ ratbag_device_new(struct ratbag *ratbag, struct udev_device *udev_device,
 
 	list_insert(&ratbag->devices, &device->link);
 
-	/* We assume that most devices have this capability, so let's set it
-	 * by default. The few devices that miss this capability should
-	 * unset it instead.
-	 */
-	ratbag_device_set_capability(device,
-				     RATBAG_DEVICE_CAP_QUERY_CONFIGURATION);
-
 	return device;
 }
 
@@ -248,6 +241,8 @@ ratbag_sanity_check_device(struct ratbag_device *device)
 
 	ratbag_device_for_each_profile(device, profile) {
 		struct ratbag_resolution *resolution;
+		unsigned int vals[300];
+		unsigned int nvals = ARRAY_LENGTH(vals);
 
 		/* Allow max 1 active profile */
 		if (profile->is_active) {
@@ -270,12 +265,6 @@ ratbag_sanity_check_device(struct ratbag_device *device)
 		}
 
 		ratbag_profile_for_each_resolution(profile, resolution) {
-			unsigned int vals[300];
-			unsigned int nvals = ARRAY_LENGTH(vals);
-
-			if (!ratbag_device_has_capability(device, RATBAG_DEVICE_CAP_RESOLUTION))
-				break;
-
 			nvals = ratbag_resolution_get_dpi_list(resolution, vals, nvals);
 			if (nvals == 0) {
 				log_bug_libratbag(ratbag,
@@ -284,13 +273,14 @@ ratbag_sanity_check_device(struct ratbag_device *device)
 				goto out;
 			}
 
-			nvals = ratbag_resolution_get_report_rate_list(resolution, vals, nvals);
-			if (nvals == 0) {
-				log_bug_libratbag(ratbag,
-						  "%s: invalid report rate list\n",
-						  device->name);
-				goto out;
-			}
+		}
+
+		nvals = ratbag_profile_get_report_rate_list(profile, vals, nvals);
+		if (nvals == 0) {
+			log_bug_libratbag(ratbag,
+					  "%s: invalid report rate list\n",
+					  device->name);
+			goto out;
 		}
 	}
 
@@ -417,37 +407,29 @@ ratbag_assign_driver(struct ratbag_device *device,
 	return rc;
 }
 
-static int
-get_device_name(struct udev_device *device,
-		char *name, size_t sz)
+static char *
+get_device_name(struct udev_device *device)
 {
 	const char *prop;
 
-	prop = udev_prop_value(device, "NAME");
-	if (!prop)
-		return -EINVAL;
+	prop = udev_prop_value(device, "HID_NAME");
 
-	/* udev name is enclosed by " */
-	sz = min(sz, strlen(prop) - 1);
-	snprintf(name, sz, "%s", &prop[1]);
-
-	return 0;
+	return strdup_safe(prop);
 }
 
 static inline int
 get_product_id(struct udev_device *device, struct input_id *id)
 {
 	const char *product;
-	struct input_id ids;
+	struct input_id ids  = {0};
 	int rc;
 
-	product = udev_prop_value(device, "PRODUCT");
+	product = udev_prop_value(device, "HID_ID");
 	if (!product)
 		return -1;
 
-	rc = sscanf(product, "%hx/%hx/%hx/%hx", &ids.bustype,
-		    &ids.vendor, &ids.product, &ids.version);
-	if (rc != 4)
+	rc = sscanf(product, "%hx:%hx:%hx", &ids.bustype, &ids.vendor, &ids.product);
+	if (rc != 3)
 		return -1;
 
 	*id = ids;
@@ -461,20 +443,17 @@ ratbag_device_new_from_udev_device(struct ratbag *ratbag,
 {
 	struct ratbag_device *device = NULL;
 	enum ratbag_error_code error = RATBAG_ERROR_DEVICE;
-	char name[128];
+	_cleanup_free_ char *name = NULL;
 	struct input_id id;
 
 	assert(ratbag != NULL);
 	assert(udev_device != NULL);
 	assert(device_out != NULL);
 
-	if (udev_prop_value(udev_device, "ID_INPUT_MOUSE") == NULL)
-		goto out_err;
-
 	if (get_product_id(udev_device, &id) != 0)
 		goto out_err;
 
-	if (get_device_name(udev_device, name, sizeof(name)) != 0)
+	if ((name = get_device_name(udev_device)) == 0)
 		goto out_err;
 
 	device = ratbag_device_new(ratbag, udev_device, name, &id);
@@ -526,16 +505,36 @@ ratbag_device_get_name(const struct ratbag_device* device)
 }
 
 LIBRATBAG_EXPORT const char *
-ratbag_device_get_svg_name(const struct ratbag_device* device)
+ratbag_device_get_bustype(const struct ratbag_device *device)
 {
-	const char *name = NULL;
+	switch (device->ids.bustype) {
+	case BUS_USB:
+		return "usb";
+	case BUS_BLUETOOTH:
+		return "bluetooth";
+	default:
+		return NULL;
+	}
+}
 
-	if (device->data)
-		name = ratbag_device_data_get_svg(device->data);
-	if (!name && device->driver->get_svg_name)
-		name = device->driver->get_svg_name(device);
+LIBRATBAG_EXPORT uint32_t
+ratbag_device_get_vendor_id(const struct ratbag_device *device)
+{
+	return device->ids.vendor;
+}
 
-	return name;
+LIBRATBAG_EXPORT uint32_t
+ratbag_device_get_product_id(const struct ratbag_device *device)
+{
+	return device->ids.product;
+}
+
+LIBRATBAG_EXPORT uint32_t
+ratbag_device_get_product_version(const struct ratbag_device *device)
+{
+	/* change this when we have a need for it, i.e. when we start supporting devices
+	 * where the USB ID gets reused */
+	return 0;
 }
 
 void
@@ -692,7 +691,7 @@ ratbag_create_profile(struct ratbag_device *device,
 	profile->index = index;
 	profile->num_resolutions = 0;
 	profile->is_enabled = true;
-	profile->name = asprintf_safe("Profile %d", index + 1);
+	profile->name = NULL;
 
 	list_insert(&device->profiles, &profile->link);
 	list_init(&profile->buttons);
@@ -729,24 +728,6 @@ ratbag_device_init_profiles(struct ratbag_device *device,
 
 	device->num_profiles = num_profiles;
 
-	if (num_profiles > 1) {
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_PROFILE);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_SWITCHABLE_PROFILE);
-
-		/* having more than one profile means we can remap the buttons
-		 * at least */
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON_KEY);
-	}
-
-	if (num_resolutions > 1) {
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_RESOLUTION);
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_SWITCHABLE_RESOLUTION);
-	}
-
-	if (num_leds > 0)
-		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_LED);
-
 	return 0;
 }
 
@@ -822,8 +803,7 @@ ratbag_device_get_profile(struct ratbag_device *device, unsigned int index)
 LIBRATBAG_EXPORT enum ratbag_error_code
 ratbag_profile_set_enabled(struct ratbag_profile *profile, bool enabled)
 {
-	if (!ratbag_device_has_capability(profile->device,
-					  RATBAG_DEVICE_CAP_DISABLE_PROFILE))
+	if (!ratbag_profile_has_capability(profile, RATBAG_PROFILE_CAP_DISABLE))
 		return RATBAG_ERROR_CAPABILITY;
 
 	profile->is_enabled = enabled;
@@ -862,47 +842,6 @@ ratbag_device_get_num_leds(struct ratbag_device *device)
 	return device->num_leds;
 }
 
-void
-ratbag_device_set_capability(struct ratbag_device *device,
-			     enum ratbag_device_capability cap)
-{
-	if (cap == RATBAG_DEVICE_CAP_NONE || cap >= MAX_CAP)
-		abort();
-
-	if (cap != RATBAG_DEVICE_CAP_QUERY_CONFIGURATION && cap % 100) {
-		enum ratbag_device_capability parent_cap = (cap/100) * 100;
-
-		if (!long_bit_is_set(device->capabilities, parent_cap)) {
-			log_bug_libratbag(device->ratbag,
-					  "Cap %d requires setting %d\n",
-					  cap, parent_cap);
-			abort();
-		}
-	}
-
-	long_set_bit(device->capabilities, cap);
-}
-
-void
-ratbag_device_unset_capability(struct ratbag_device *device,
-			     enum ratbag_device_capability cap)
-{
-	if (cap == RATBAG_DEVICE_CAP_NONE || cap >= MAX_CAP)
-		abort();
-
-	long_clear_bit(device->capabilities, cap);
-}
-
-LIBRATBAG_EXPORT bool
-ratbag_device_has_capability(const struct ratbag_device *device,
-			     enum ratbag_device_capability cap)
-{
-	if (cap == RATBAG_DEVICE_CAP_NONE || cap >= MAX_CAP)
-		abort();
-
-	return long_bit_is_set(device->capabilities, cap);
-}
-
 static inline enum ratbag_error_code
 write_led_helper(struct ratbag_device *device, struct ratbag_led *led)
 {
@@ -1015,8 +954,8 @@ ratbag_profile_set_active(struct ratbag_profile *profile)
 	struct ratbag_profile *p;
 	int rc;
 
-	if (!ratbag_device_has_capability(device, RATBAG_DEVICE_CAP_SWITCHABLE_PROFILE))
-		return RATBAG_ERROR_CAPABILITY;
+	if (device->num_profiles == 1)
+		return RATBAG_SUCCESS;
 
 	assert(device->driver->set_active_profile);
 	rc = device->driver->set_active_profile(device, profile->index);
@@ -1089,7 +1028,6 @@ ratbag_resolution_has_capability(struct ratbag_resolution *resolution,
 				 enum ratbag_resolution_capability cap)
 {
 	switch (cap) {
-	case RATBAG_RESOLUTION_CAP_INDIVIDUAL_REPORT_RATE:
 	case RATBAG_RESOLUTION_CAP_SEPARATE_XY_RESOLUTION:
 		break;
 	default:
@@ -1099,12 +1037,27 @@ ratbag_resolution_has_capability(struct ratbag_resolution *resolution,
 	return !!(resolution->capabilities & (1 << cap));
 }
 
+static inline bool
+resolution_has_dpi(struct ratbag_resolution *resolution,
+		   unsigned int dpi)
+{
+	for (size_t i = 0; i < resolution->ndpis; i++) {
+		if (dpi == resolution->dpis[i])
+			return true;
+	}
+
+	return false;
+}
+
 LIBRATBAG_EXPORT enum ratbag_error_code
 ratbag_resolution_set_dpi(struct ratbag_resolution *resolution,
 			  unsigned int dpi)
 {
 	struct ratbag_profile *profile = resolution->profile;
 
+	if (!resolution_has_dpi(resolution, dpi))
+		return RATBAG_ERROR_VALUE;
+
 	if (resolution->dpi_x != dpi || resolution->dpi_y != dpi) {
 		resolution->dpi_x = dpi;
 		resolution->dpi_y = dpi;
@@ -1128,6 +1081,9 @@ ratbag_resolution_set_dpi_xy(struct ratbag_resolution *resolution,
 	if ((x == 0 && y != 0) || (x != 0 && y == 0))
 		return RATBAG_ERROR_VALUE;
 
+	if (!resolution_has_dpi(resolution, x) || !resolution_has_dpi(resolution, y))
+		return RATBAG_ERROR_VALUE;
+
 	if (resolution->dpi_x != x || resolution->dpi_y != y) {
 		resolution->dpi_x = x;
 		resolution->dpi_y = y;
@@ -1139,29 +1095,12 @@ ratbag_resolution_set_dpi_xy(struct ratbag_resolution *resolution,
 }
 
 LIBRATBAG_EXPORT enum ratbag_error_code
-ratbag_resolution_set_report_rate(struct ratbag_resolution *resolution,
-				  unsigned int hz)
+ratbag_profile_set_report_rate(struct ratbag_profile *profile,
+			       unsigned int hz)
 {
-	if (ratbag_resolution_has_capability(resolution,
-					      RATBAG_RESOLUTION_CAP_INDIVIDUAL_REPORT_RATE)) {
-		if (resolution->hz != hz) {
-			resolution->hz = hz;
-			resolution->dirty = true;
-			resolution->profile->dirty = true;
-		}
-	} else {
-		struct ratbag_profile *profile = resolution->profile;
-		struct ratbag_resolution *res;
-
-		/* No indvidual report rate per resolution. Loop through all of
-		 * them and update. */
-		ratbag_profile_for_each_resolution(profile, res) {
-			if (res->hz != hz) {
-				res->hz = hz;
-				res->dirty = true;
-				profile->dirty = true;
-			}
-		}
+	if (profile->hz != hz) {
+		profile->hz = hz;
+		profile->dirty = true;
 	}
 
 	return RATBAG_SUCCESS;
@@ -1204,27 +1143,27 @@ ratbag_resolution_get_dpi_y(struct ratbag_resolution *resolution)
 }
 
 LIBRATBAG_EXPORT int
-ratbag_resolution_get_report_rate(struct ratbag_resolution *resolution)
+ratbag_profile_get_report_rate(struct ratbag_profile *profile)
 {
-	return resolution->hz;
+	return profile->hz;
 }
 
 LIBRATBAG_EXPORT size_t
-ratbag_resolution_get_report_rate_list(struct ratbag_resolution *resolution,
-				       unsigned int *rates,
-				       size_t nrates)
+ratbag_profile_get_report_rate_list(struct ratbag_profile *profile,
+				    unsigned int *rates,
+				    size_t nrates)
 {
-	_Static_assert(sizeof(*rates) == sizeof(*resolution->rates), "type mismatch");
+	_Static_assert(sizeof(*rates) == sizeof(*profile->rates), "type mismatch");
 
 	assert(nrates > 0);
 
-	if (resolution->nrates == 0)
+	if (profile->nrates == 0)
 		return 0;
 
-	memcpy(rates, resolution->rates,
-	       sizeof(unsigned int) * min(nrates, resolution->nrates));
+	memcpy(rates, profile->rates,
+	       sizeof(unsigned int) * min(nrates, profile->nrates));
 
-	return resolution->nrates;
+	return profile->nrates;
 }
 
 LIBRATBAG_EXPORT bool
@@ -1303,12 +1242,6 @@ ratbag_profile_get_button(struct ratbag_profile *profile,
 	return NULL;
 }
 
-LIBRATBAG_EXPORT enum ratbag_button_type
-ratbag_button_get_type(struct ratbag_button *button)
-{
-	return button->type;
-}
-
 LIBRATBAG_EXPORT enum ratbag_button_action_type
 ratbag_button_get_action_type(struct ratbag_button *button)
 {
@@ -1357,8 +1290,8 @@ ratbag_button_set_button(struct ratbag_button *button, unsigned int btn)
 {
 	struct ratbag_button_action action = {0};
 
-	if (!ratbag_device_has_capability(button->profile->device,
-					  RATBAG_DEVICE_CAP_BUTTON_KEY))
+	if (!ratbag_button_has_action_type(button,
+					   RATBAG_BUTTON_ACTION_TYPE_BUTTON))
 		return RATBAG_ERROR_CAPABILITY;
 
 	action.type = RATBAG_BUTTON_ACTION_TYPE_BUTTON;
@@ -1388,8 +1321,8 @@ ratbag_button_set_special(struct ratbag_button *button,
 
 	/* FIXME: range checks */
 
-	if (!ratbag_device_has_capability(button->profile->device,
-					  RATBAG_DEVICE_CAP_BUTTON_KEY))
+	if (!ratbag_button_has_action_type(button,
+					   RATBAG_BUTTON_ACTION_TYPE_SPECIAL))
 		return RATBAG_ERROR_CAPABILITY;
 
 	action.type = RATBAG_BUTTON_ACTION_TYPE_SPECIAL;
@@ -1425,8 +1358,8 @@ ratbag_button_set_key(struct ratbag_button *button,
 
 	/* FIXME: range checks */
 
-	if (!ratbag_device_has_capability(button->profile->device,
-					  RATBAG_DEVICE_CAP_BUTTON_KEY))
+	if (!ratbag_button_has_action_type(button,
+					   RATBAG_BUTTON_ACTION_TYPE_KEY))
 		return RATBAG_ERROR_CAPABILITY;
 
 	/* FIXME: modifiers */
@@ -1446,8 +1379,8 @@ ratbag_button_disable(struct ratbag_button *button)
 {
 	struct ratbag_button_action action = {0};
 
-	if (!ratbag_device_has_capability(button->profile->device,
-					  RATBAG_DEVICE_CAP_BUTTON_KEY))
+	if (!ratbag_button_has_action_type(button,
+					   RATBAG_BUTTON_ACTION_TYPE_KEY))
 		return RATBAG_ERROR_CAPABILITY;
 
 	action.type = RATBAG_BUTTON_ACTION_TYPE_NONE;
@@ -1539,12 +1472,6 @@ ratbag_led_get_mode(struct ratbag_led *led)
 	return led->mode;
 }
 
-LIBRATBAG_EXPORT enum ratbag_led_type
-ratbag_led_get_type(struct ratbag_led *led)
-{
-	return led->type;
-}
-
 LIBRATBAG_EXPORT bool
 ratbag_led_has_mode(struct ratbag_led *led,
 		    enum ratbag_led_mode mode)
@@ -1578,11 +1505,6 @@ ratbag_led_get_brightness(struct ratbag_led *led)
 LIBRATBAG_EXPORT enum ratbag_error_code
 ratbag_led_set_mode(struct ratbag_led *led, enum ratbag_led_mode mode)
 {
-
-	if (!ratbag_device_has_capability(led->profile->device,
-					  RATBAG_DEVICE_CAP_LED))
-		return RATBAG_ERROR_CAPABILITY;
-
 	led->mode = mode;
 	led->dirty = true;
 	led->profile->dirty = true;
@@ -1592,10 +1514,6 @@ ratbag_led_set_mode(struct ratbag_led *led, enum ratbag_led_mode mode)
 LIBRATBAG_EXPORT enum ratbag_error_code
 ratbag_led_set_color(struct ratbag_led *led, struct ratbag_color color)
 {
-	if (!ratbag_device_has_capability(led->profile->device,
-					  RATBAG_DEVICE_CAP_LED))
-		return RATBAG_ERROR_CAPABILITY;
-
 	led->color = color;
 	led->dirty = true;
 	led->profile->dirty = true;
@@ -1605,20 +1523,12 @@ ratbag_led_set_color(struct ratbag_led *led, struct ratbag_color color)
 LIBRATBAG_EXPORT enum ratbag_led_colordepth
 ratbag_led_get_colordepth(struct ratbag_led *led)
 {
-	if (!ratbag_device_has_capability(led->profile->device,
-					  RATBAG_DEVICE_CAP_LED))
-		return RATBAG_ERROR_CAPABILITY;
-
 	return led->colordepth;
 }
 
 LIBRATBAG_EXPORT enum ratbag_error_code
 ratbag_led_set_effect_duration(struct ratbag_led *led, unsigned int ms)
 {
-	if (!ratbag_device_has_capability(led->profile->device,
-					  RATBAG_DEVICE_CAP_LED))
-		return RATBAG_ERROR_CAPABILITY;
-
 	led->ms = ms;
 	led->dirty = true;
 	led->profile->dirty = true;
@@ -1628,10 +1538,6 @@ ratbag_led_set_effect_duration(struct ratbag_led *led, unsigned int ms)
 LIBRATBAG_EXPORT enum ratbag_error_code
 ratbag_led_set_brightness(struct ratbag_led *led, unsigned int brightness)
 {
-	if (!ratbag_device_has_capability(led->profile->device,
-					  RATBAG_DEVICE_CAP_LED))
-		return RATBAG_ERROR_CAPABILITY;
-
 	led->brightness = brightness;
 	led->dirty = true;
 	led->profile->dirty = true;
@@ -1673,8 +1579,7 @@ ratbag_profile_set_name(struct ratbag_profile *profile,
 {
 	char *name_copy;
 
-	if (!ratbag_profile_has_capability(profile,
-					   RATBAG_PROFILE_CAP_WRITABLE_NAME))
+	if (!profile->name)
 		return RATBAG_ERROR_CAPABILITY;
 
 	name_copy = strdup_safe(name);
@@ -1787,8 +1692,8 @@ LIBRATBAG_EXPORT enum ratbag_error_code
 ratbag_button_set_macro(struct ratbag_button *button,
 			const struct ratbag_button_macro *macro)
 {
-	if (!ratbag_device_has_capability(button->profile->device,
-					  RATBAG_DEVICE_CAP_BUTTON_MACROS))
+	if (!ratbag_button_has_action_type(button,
+					   RATBAG_BUTTON_ACTION_TYPE_MACRO))
 		return RATBAG_ERROR_CAPABILITY;
 
 	ratbag_button_copy_macro(button, macro);
